import re
import os
import io
import sys
import glob
import yaml
import json
import lxml
import shutil
import zipfile
import logging
from lxml import etree
from boldqc.bids import BIDS
import boldqc.parsers as parsers

logger = logging.getLogger(__name__)

class Report:
    def __init__(self, bids, sub, ses, run):
        self.module = os.path.dirname(__file__)
        self.bids = bids
        self.sub = sub
        self.run = run
        self.ses = ses if ses else ''
 
    def getdirs(self):
        self.dirs = {
            'boldqc': None
        }
        d = os.path.join(
            self.bids,
            'derivatives',
            'boldqc',
            'sub-' + self.sub.replace('sub-', ''),
            'ses-' + self.ses.replace('ses-', ''),
            'func'
        )
        basename = BIDS.basename(**{
            'sub': self.sub,
            'ses': self.ses,
            'run': self.run,
            'mod': 'bold'
        })
        dirname = os.path.join(d, basename)
        logger.debug('looking for %s', dirname)
        if os.path.exists(dirname):
            self.dirs['boldqc'] = dirname
        logger.debug('boldqc dir: %s', self.dirs['boldqc'])

    def build_assessment(self, output):
        '''
        Build XNAT assessment

        :param output: Base output directory
        '''
        basename = BIDS.basename(**{
            'sub': self.sub,
            'ses': self.ses,
            'run': self.run,
            'mod': 'bold'
        })
        self.getdirs()
        if not self.dirs['boldqc']:
            raise AssessmentError('need boldqc data to build assessment')

        # initialize namespaces
        ns = {
            None: 'http://www.neuroinfo.org/neuroinfo',
            'xs': 'http://www.w3.org/2001/XMLSchema',
            'xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xnat': 'http://nrg.wustl.edu/xnat',
            'neuroinfo': 'http://www.neuroinfo.org/neuroinfo'
        }

        # read json sidecar for scan number
        boldqc_ds = self.datasource('boldqc')
        logger.info('boldqc info %s', '|'.join(boldqc_ds.values()))

        # assessment id
        aid = '{0}_BOLD_{1}_EQC'.format(
            boldqc_ds['experiment'],
            boldqc_ds['scan']
        )
        logger.info('Assessor ID %s', aid)

        # root element
        xnatns = '{%s}' % ns['xnat']
        root = etree.Element('BOLDQC', nsmap=ns)
        root.attrib['project'] = boldqc_ds['project']
        root.attrib['ID'] = aid
        root.attrib['label'] = aid
        # get start date and time from morph provenance
        fname = os.path.join(self.dirs['boldqc'], 'logs', 'provenance.json')
        with open(fname) as fo:
            prov = json.load(fo)
        # add date and time
        etree.SubElement(root, xnatns + 'date').text = prov['start_date']
        etree.SubElement(root, xnatns + 'time').text = prov['start_time']
        # compile a list of files to be added to xnat:out section
        resources = [
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_mean.nii.gz'),
                'dest': os.path.join('mean-nifti', f'{aid}_mean.nii.gz')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_mean_thumbnail.png'),
                'dest': os.path.join('mean-image', f'{aid}_mean_thumbnail.png')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_mean_slice.txt'),
                'dest': os.path.join('mean-slice-data', f'{aid}_mean_slice.txt')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_mean_slice.png'),
                'dest': os.path.join('mean-slice-image', f'{aid}_mean_slice.png')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_mask.nii.gz'),
                'dest': os.path.join('mask-nifti', f'{aid}_mask.nii.gz')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_mask_thumbnail.png'),
                'dest': os.path.join('mask-image', f'{aid}_mask_thumbnail.png')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_snr.nii.gz'),
                'dest': os.path.join('snr-nifti', f'{aid}_snr.nii.gz')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_snr_thumbnail.png'),
                'dest': os.path.join('snr-image', f'{aid}_snr_thumbnail.png')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_stdev.nii.gz'),
                'dest': os.path.join('stdev-nifti', f'{aid}_stdev.nii.gz')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_stdev_thumbnail.png'),
                'dest': os.path.join('stdev-image', f'{aid}_stdev_thumbnail.png')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_motion.png'),
                'dest': os.path.join('motion-image', f'{aid}_motion.png')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_slope.nii.gz'),
                'dest': os.path.join('slope-nifti', f'{aid}_slope.nii.gz')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_slope_thumbnail.png'),
                'dest': os.path.join('slope-image', f'{aid}_slope_thumbnail.png')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_auto_report.txt'),
                'dest': os.path.join('auto-report', f'{aid}_auto_report.txt')
            },
            {
                'source': os.path.join(self.dirs['boldqc'], f'{basename}_slice_report.txt'),
                'dest': os.path.join('slice-report', f'{aid}_slice_report.txt')
            }
        ]

        floatfmt = lambda x: '{:f}'.format(float(x))

        # parse auto_report
        auto_report_file = os.path.join(self.dirs['boldqc'], f'{basename}_auto_report.txt')
        auto_report = parsers.parse_auto_report(auto_report_file)
        # parse slice_report
        slice_report_file = os.path.join(self.dirs['boldqc'], f'{basename}_slice_report.txt')
        slice_report = parsers.parse_slice_report(slice_report_file)
        # start building XML
        xnatns = '{%s}' % ns['xnat']
        etree.SubElement(root, xnatns + 'imageSession_ID').text = boldqc_ds['experiment_id']
        etree.SubElement(root, 'bold_scan_id').text = boldqc_ds['scan']
        etree.SubElement(root, 'session_label').text = boldqc_ds['experiment']
        etree.SubElement(root, 'Size').text = auto_report['InputFileSize']
        etree.SubElement(root, 'N_Vols').text = auto_report['N_Vols']
        etree.SubElement(root, 'Skip').text = auto_report['Skip']
        etree.SubElement(root, 'qc_N_Tps').text = auto_report['qc_N_Tps']
        etree.SubElement(root, 'qc_thresh').text = floatfmt(auto_report['qc_thresh'])
        etree.SubElement(root, 'qc_nVox').text = auto_report['qc_nVox']
        etree.SubElement(root, 'qc_Mean').text = floatfmt(auto_report['qc_Mean'])
        etree.SubElement(root, 'qc_Max').text = floatfmt(slice_report['qc_Max'])
        etree.SubElement(root, 'qc_Min').text = floatfmt(slice_report['qc_Min'])
        etree.SubElement(root, 'qc_Stdev').text = floatfmt(auto_report['qc_Stdev'])
        etree.SubElement(root, 'qc_sSNR').text = floatfmt(auto_report['qc_sSNR'])
        etree.SubElement(root, 'qc_vSNR').text = floatfmt(auto_report['qc_vSNR'])
        etree.SubElement(root, 'qc_slope').text = floatfmt(auto_report['qc_slope'])
        etree.SubElement(root, 'mot_N_Tps').text = auto_report['mot_N_Tps']
        etree.SubElement(root, 'mot_rel_x_mean').text = floatfmt(auto_report['mot_rel_x_mean'])
        etree.SubElement(root, 'mot_rel_x_sd').text = floatfmt(auto_report['mot_rel_x_sd'])
        etree.SubElement(root, 'mot_rel_x_max').text = floatfmt(auto_report['mot_rel_x_max'])
        etree.SubElement(root, 'mot_rel_x_1mm').text = auto_report['mot_rel_x_1mm']
        etree.SubElement(root, 'mot_rel_x_5mm').text = auto_report['mot_rel_x_5mm']
        etree.SubElement(root, 'mot_rel_y_mean').text = floatfmt(auto_report['mot_rel_y_mean'])
        etree.SubElement(root, 'mot_rel_y_sd').text = floatfmt(auto_report['mot_rel_y_sd'])
        etree.SubElement(root, 'mot_rel_y_max').text = floatfmt(auto_report['mot_rel_y_max'])
        etree.SubElement(root, 'mot_rel_y_1mm').text = auto_report['mot_rel_y_1mm']
        etree.SubElement(root, 'mot_rel_y_5mm').text = auto_report['mot_rel_y_5mm']
        etree.SubElement(root, 'mot_rel_z_mean').text = floatfmt(auto_report['mot_rel_z_mean'])
        etree.SubElement(root, 'mot_rel_z_sd').text = floatfmt(auto_report['mot_rel_z_sd'])
        etree.SubElement(root, 'mot_rel_z_max').text = floatfmt(auto_report['mot_rel_z_max'])
        etree.SubElement(root, 'mot_rel_z_1mm').text = auto_report['mot_rel_z_1mm']
        etree.SubElement(root, 'mot_rel_z_5mm').text = auto_report['mot_rel_z_5mm']
        etree.SubElement(root, 'mot_rel_xyz_mean').text = floatfmt(auto_report['mot_rel_xyz_mean'])
        etree.SubElement(root, 'mot_rel_xyz_sd').text = floatfmt(auto_report['mot_rel_xyz_sd'])
        etree.SubElement(root, 'mot_rel_xyz_max').text = floatfmt(auto_report['mot_rel_xyz_max'])
        etree.SubElement(root, 'mot_rel_xyz_1mm').text = auto_report['mot_rel_xyz_1mm']
        etree.SubElement(root, 'mot_rel_xyz_5mm').text = auto_report['mot_rel_xyz_5mm']
        etree.SubElement(root, 'rot_rel_x_mean').text = floatfmt(auto_report['rot_rel_x_mean'])
        etree.SubElement(root, 'rot_rel_x_sd').text = floatfmt(auto_report['rot_rel_x_sd'])
        etree.SubElement(root, 'rot_rel_x_max').text = floatfmt(auto_report['rot_rel_x_max'])
        etree.SubElement(root, 'rot_rel_y_mean').text = floatfmt(auto_report['rot_rel_y_mean'])
        etree.SubElement(root, 'rot_rel_y_sd').text = floatfmt(auto_report['rot_rel_y_sd'])
        etree.SubElement(root, 'rot_rel_y_max').text = floatfmt(auto_report['rot_rel_y_max'])
        etree.SubElement(root, 'rot_rel_z_mean').text = floatfmt(auto_report['rot_rel_z_mean'])
        etree.SubElement(root, 'rot_rel_z_sd').text = floatfmt(auto_report['rot_rel_z_sd'])
        etree.SubElement(root, 'rot_rel_z_max').text = floatfmt(auto_report['rot_rel_z_max'])
        etree.SubElement(root, 'mot_abs_x_mean').text = floatfmt(auto_report['mot_abs_x_mean'])
        etree.SubElement(root, 'mot_abs_x_sd').text = floatfmt(auto_report['mot_abs_x_sd'])
        etree.SubElement(root, 'mot_abs_x_max').text = floatfmt(auto_report['mot_abs_x_max'])
        etree.SubElement(root, 'mot_abs_y_mean').text = floatfmt(auto_report['mot_abs_y_mean'])
        etree.SubElement(root, 'mot_abs_y_sd').text = floatfmt(auto_report['mot_abs_y_sd'])
        etree.SubElement(root, 'mot_abs_y_max').text = floatfmt(auto_report['mot_abs_y_max'])
        etree.SubElement(root, 'mot_abs_z_mean').text = floatfmt(auto_report['mot_abs_z_mean'])
        etree.SubElement(root, 'mot_abs_z_sd').text = floatfmt(auto_report['mot_abs_z_sd'])
        etree.SubElement(root, 'mot_abs_z_max').text = floatfmt(auto_report['mot_abs_z_max'])
        etree.SubElement(root, 'mot_abs_xyz_mean').text = floatfmt(auto_report['mot_abs_xyz_mean'])
        etree.SubElement(root, 'mot_abs_xyz_sd').text = floatfmt(auto_report['mot_abs_xyz_sd'])
        etree.SubElement(root, 'mot_abs_xyz_max').text = floatfmt(auto_report['mot_abs_xyz_max'])
        etree.SubElement(root, 'rot_abs_x_mean').text = floatfmt(auto_report['rot_abs_x_mean'])
        etree.SubElement(root, 'rot_abs_x_sd').text = floatfmt(auto_report['rot_abs_x_sd'])
        etree.SubElement(root, 'rot_abs_x_max').text = floatfmt(auto_report['rot_abs_x_max'])
        etree.SubElement(root, 'rot_abs_y_mean').text = floatfmt(auto_report['rot_abs_y_mean'])
        etree.SubElement(root, 'rot_abs_y_sd').text = floatfmt(auto_report['rot_abs_y_sd'])
        etree.SubElement(root, 'rot_abs_y_max').text = floatfmt(auto_report['rot_abs_y_max'])
        etree.SubElement(root, 'rot_abs_z_mean').text = floatfmt(auto_report['rot_abs_z_mean'])
        etree.SubElement(root, 'rot_abs_z_sd').text = floatfmt(auto_report['rot_abs_z_sd'])
        etree.SubElement(root, 'rot_abs_z_max').text = floatfmt(auto_report['rot_abs_z_max'])

        # write assessor to output mount location.
        xmlstr = etree.tostring(root, pretty_print=True, xml_declaration=True, encoding='UTF-8')
        assessor_dir = os.path.join(output, 'assessor')
        os.makedirs(assessor_dir, exist_ok=True)
        assessment_xml = os.path.join(assessor_dir, 'assessment.xml')
        logger.debug(f'writing {assessment_xml}')
        with open(assessment_xml, 'wb') as fo:
            fo.write(xmlstr)

        # copy resources to output mount location
        resources_dir = os.path.join(output, 'resources')
        os.makedirs(resources_dir, exist_ok=True)
        logger.debug(f'copying resources into {resources_dir}')
        for resource in resources:
            src = resource['source']
            dest = os.path.join(resources_dir, resource['dest'])
            os.makedirs(os.path.dirname(dest), exist_ok=True)
            shutil.copyfile(src, dest)

    def datasource(self, task):
        basename = os.path.basename(self.dirs[task])
        sidecar = os.path.join(self.dirs[task], 'logs', basename + '.json')
        if not os.path.exists(sidecar):
            raise FileNotFoundError(sidecar)
        with open(sidecar) as fo:
            js = json.load(fo)
        return js['DataSource']['application/x-xnat']

    def protocol(self, task):
        basename = os.path.basename(self.dirs[task])
        sidecar = os.path.join(self.dirs[task], 'logs', basename + '.json')
        if not os.path.exists(sidecar):
            raise FileNotFoundError(sidecar)
        with open(sidecar) as fo:
            js = json.load(fo)
        return js['ProtocolName']

class AssessmentError(Exception):
    pass



import os
import re
import sys
import json
import yaml
import yaxil
import logging
import argparse as ap
import subprocess as sp
import collections as col

logger = logging.getLogger(__name__)

def do(args):
    if args.insecure:
        logger.warning('disabling ssl certificate verification')
        yaxil.CHECK_CERTIFICATE = False

    # load authentication data and set environment variables for ArcGet.py
    auth = yaxil.auth2(
        args.xnat_alias,
        args.xnat_host,
        args.xnat_user,
        args.xnat_pass
    )
    os.environ['XNAT_HOST'] = auth.url
    os.environ['XNAT_USER'] = auth.username
    os.environ['XNAT_PASS'] = auth.password

    # query BOLD scans
    with yaxil.session(auth) as ses:
        scans = col.defaultdict(dict)
        for scan in ses.scans(label=args.label, project=args.project):
            note = scan['note']
            bold_match = re.match('.*(^|\s)#BOLD(?P<run>_\d+)?(\s|$).*', note, flags=re.IGNORECASE)
            if bold_match:
                run = bold_match.group('run')
                run = re.sub('[^0-9]', '', run or '1')
                run = int(run)
                scans[run]['bold'] = scan['id']
    logger.info(json.dumps(scans, indent=2))
    for run,scansr in scans.items():
        logger.info('getting bold run=%s, scan=%s', run, scansr['bold'])
        get_bold(args, auth, run, scansr['bold'], verbose=args.verbose)

def get_bold(args, auth, run, scan, verbose=False):
    config = {
        'func': {
            'bold': [
                {
                    'run': int(run),
                    'scan': scan
                }
            ]
        }
    }
    config = yaml.safe_dump(config)
    cmd = [
        'ArcGet.py',
        '--label', args.label,
        '--output-dir', args.bids_dir,
        '--output-format', 'bids',
    ]
    if args.project:
        cmd.extend([
            '--project', args.project
        ])
    if args.insecure:
        cmd.extend([
            '--insecure'
        ])
    if args.in_mem:
        cmd.extend([
            '--in-mem'
        ])
    cmd.extend([
        '--config', '-'
    ])
    if verbose:
        cmd.append('--debug')
    logger.info(sp.list2cmdline(cmd))
    sp.check_output(cmd, input=config.encode('utf-8'))




import os
import re
import sys
import json
import yaml
import yaxil
import glob
import math
import boldqc
import logging
import tarfile
import executors
import tempfile as tf
import subprocess as sp
import boldqc.tasks.stackcheck_ext as stackcheck_ext
import boldqc.tasks.niftiqa_wrapper as niftiqa_wrapper
from executors.models import Job, JobArray
from boldqc.bids import BIDS
from boldqc.xnat import Report
from boldqc.state import State

logger = logging.getLogger(__name__)
LIBEXEC = os.path.realpath(os.path.join(os.path.dirname(boldqc.__file__), 'libexec'))
os.environ['PATH'] = LIBEXEC + ':' + os.environ['PATH']

def do(args):
    if args.insecure:
        logger.warning('disabling ssl certificate verification')
        yaxil.CHECK_CERTIFICATE = False

    # create job executor and job array
    if args.scheduler:
        E = executors.get(args.scheduler, partition=args.partition)
    else:
        E = executors.probe(args.partition)
    jarray = JobArray(E)

    # create BIDS
    B = BIDS(args.bids_dir, args.sub, ses=args.ses)
    raw = B.raw_bold('bold', run=args.run)
    logger.debug('BOLD raw: %s', raw)

    # get repetition time from T1w sidecar for vNav processing
    sidecar = os.path.join(*raw) + '.json'
    logger.debug('sidecar: %s', sidecar)
    with open(sidecar) as fo:
        js = json.load(fo)
        tr = js['RepetitionTime']
    logger.debug('TR: %s', tr)

    boldqc_outdir = None
    infile = os.path.join(*raw) + '.nii.gz'
    boldqc_outdir = B.derivatives_dir('boldqc')
    boldqc_outdir = os.path.join(boldqc_outdir, 'func', raw[1])
    
    # niftiqa job
    task = niftiqa_wrapper.Task(
        infile,
        boldqc_outdir
    )
    logger.info(json.dumps(task.command, indent=1))
    jarray.add(task.job)

    # stackcheck_ext job
    task = stackcheck_ext.Task(
        infile,
        boldqc_outdir
    )
    logger.info(json.dumps(task.command, indent=1))
    jarray.add(task.job)

    # submit jobs and wait for them to finish
    if not args.dry_run:
        logger.info('submitting jobs')
        jarray.submit(limit=1)
        logger.info('waiting for all jobs to finish')
        jarray.wait()
        numjobs = len(jarray.array)
        failed = len(jarray.failed)
        complete = len(jarray.complete)
        if failed:
            logger.info('%s/%s jobs failed', failed, numjobs)
            for pid,job in iter(jarray.failed.items()):
                logger.error('%s exited with returncode %s', job.name, job.returncode)
                with open(job.output, 'r') as fp:
                    logger.error('standard output\n%s', fp.read())
                with open(job.error, 'r') as fp:
                    logger.error('standard error\n%s', fp.read())
        logger.info('%s/%s jobs completed', complete, numjobs)
        if failed > 0:
            sys.exit(1)

    # artifacts directory
    if not args.artifacts_dir:
        args.artifacts_dir = os.path.join(
            boldqc_outdir,
            'xnat-artifacts'
        )

    # build data to upload to XNAT
    R = Report(args.bids_dir, args.sub, args.ses, args.run)
    logger.info('building xnat artifacts to %s', args.artifacts_dir)
    R.build_assessment(args.artifacts_dir)

    # upload data to xnat over rest api
    if args.xnat_upload:
        logger.info('Uploading artifacts to XNAT')
        auth = yaxil.auth2(args.xnat_alias)
        yaxil.storerest(auth, args.artifacts_dir, 'boldqc-resource')




#!/usr/bin/env python

import boldqc
import logging
import argparse as ap
import boldqc.cli as cli

logger = logging.getLogger(__name__)

def main():
    parser = ap.ArgumentParser()
    parser.add_argument('-v', '--verbose', action='store_true',
        help='Enable verbose logging')
    parser.add_argument('--insecure', action='store_true',
        help='Disable SSL certificate verification')
    subparsers = parser.add_subparsers(help='sub-command help')
    # get mode
    parser_get = subparsers.add_parser('get', help='get -h')
    parser_get.add_argument('--in-mem', action='store_true',
        help='Download data from XNAT in memory')
    parser_get.add_argument('--label', required=True,
        help='XNAT MR Session name')
    parser_get.add_argument('--project',
        help='XNAT Project name')
    parser_get.add_argument('--bids-dir', required=True,
        help='Output BIDS directory')
    parser_get.add_argument('--xnat-alias',
        help='YAXIL authentication alias')
    parser_get.add_argument('--xnat-host',
        help='XNAT host')
    parser_get.add_argument('--xnat-user',
        help='XNAT username')
    parser_get.add_argument('--xnat-pass',
        help='XNAT password')
    parser_get.set_defaults(func=cli.get.do)
    # process mode
    parser_process = subparsers.add_parser('process', help='process -h')
    parser_process.add_argument('--partition', default='default',
        help='Job scheduler partition')
    parser_process.add_argument('--scheduler', default='local',
        help='Choose a specific job scheduler')
    parser_process.add_argument('--rate-limit', type=int, default=None, 
        help='Rate limit the number of tasks executed in parallel (1=serial)')
    parser_process.add_argument('--sub', required=True,
        help='BIDS subject')
    parser_process.add_argument('--ses',
        help='BIDS session')
    parser_process.add_argument('--mod', default='bold',
        help='BIDS modality')
    parser_process.add_argument('--run', default=1, type=int,
        help='BIDS run')
    parser_process.add_argument('--bids-dir', required=True,
        help='BIDS root directory')
    parser_process.add_argument('--dry-run', action='store_true',
        help='Do not actually execute any jobs')
    parser_process.add_argument('--xnat-alias',
        help='YAXIL authentication alias')
    parser_process.add_argument('--xnat-host',
        help='XNAT host')
    parser_process.add_argument('--xnat-user',
        help='XNAT username')
    parser_process.add_argument('--xnat-pass',
        help='XNAT password')
    parser_process.add_argument('--artifacts-dir',
        help='Location for generated XNAT assessor and resources')
    parser_process.add_argument('--xnat-upload', action='store_true',
        help='Upload results to XNAT over REST API')
    parser_process.set_defaults(func=cli.process.do)
    # tandem (run get and process)
    parser_tandem = subparsers.add_parser('tandem', help='tandem -h')
    parser_tandem.add_argument('--in-mem', action='store_true',
        help='Download data from XNAT in memory')
    parser_tandem.add_argument('--label', required=True,
        help='XNAT MR Session name')
    parser_tandem.add_argument('--project',
        help='XNAT Project name')
    parser_tandem.add_argument('--bids-dir', required=True,
        help='Output BIDS directory')
    parser_tandem.add_argument('--run', default=1, type=int,
        help='BIDS run')
    parser_tandem.add_argument('--partition', default='default',
        help='Job scheduler partition')
    parser_tandem.add_argument('--scheduler', default='local',
        help='Choose a specific job scheduler')
    parser_tandem.add_argument('--rate-limit', type=int, default=None, 
        help='Rate limit the number of tasks executed in parallel (1=serial)')
    parser_tandem.add_argument('--dry-run', action='store_true',
        help='Do not actually execute any jobs')
    parser_tandem.add_argument('--xnat-alias',
        help='YAXIL authentication alias')
    parser_tandem.add_argument('--xnat-host',
        help='XNAT host')
    parser_tandem.add_argument('--xnat-user',
        help='XNAT username')
    parser_tandem.add_argument('--xnat-pass',
        help='XNAT password')
    parser_tandem.add_argument('--artifacts-dir',
        help='Location for generated assessors and resources')
    parser_tandem.add_argument('--xnat-upload', action='store_true',
        help='Upload results to XNAT over REST API')
    parser_tandem.set_defaults(func=cli.tandem.do)
    args = parser.parse_args()

    configure_logging(args.verbose)
    logger.info('Welcome to BOLDQC version %s', boldqc.version())

    # fire parser_*.set_defaults(func=<function>)
    args.func(args)

def configure_logging(verbose):
    level = logging.INFO
    if verbose:
        level = logging.DEBUG
    logging.basicConfig(
        level=level,
        format='%(asctime)s.%(msecs)03d %(levelname)s %(module)s - %(funcName)s: %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S'
    )

if __name__ == '__main__':
    main()




import os
import re
import json
import yaml
import yaxil
import logging
import yaxil.bids
import argparse as ap
import subprocess as sp
import boldqc.cli.get
import boldqc.cli.process
import collections as col
import yaxil.bids

logger = logging.getLogger(__name__)

def do(args):
    if args.insecure:
        logger.warning('disabling ssl certificate verification')
        yaxil.CHECK_CERTIFICATE = False

    # load authentication data and set environment variables for ArcGet.py
    auth = yaxil.auth2(
        args.xnat_alias,
        args.xnat_host,
        args.xnat_user,
        args.xnat_pass
    )
    os.environ['XNAT_HOST'] = auth.url
    os.environ['XNAT_USER'] = auth.username
    os.environ['XNAT_PASS'] = auth.password

    # query BOLD scans
    with yaxil.session(auth) as ses:
        scans = col.defaultdict(dict)
        for scan in ses.scans(label=args.label, project=args.project):
            note = scan['note']
            bold_match = re.match('.*(^|\s)#BOLD(?P<run>_\d+)?(\s|$).*', note, flags=re.IGNORECASE)
            if bold_match:
                run = bold_match.group('run')
                run = re.sub('[^0-9]', '', run or '1')
                run = int(run)
                scans[run]['bold'] = scan['id']

    subject_label = scan['subject_label']
    logger.info(json.dumps(scans, indent=2))

    for run,scansr in scans.items():
        if run != args.run:
            continue
        logger.info('getting bold run=%s, scan=%s', run, scansr['bold'])
        boldqc.cli.get.get_bold(
            args,
            auth,
            run,
            scansr['bold'],
            verbose=args.verbose
        )
        args.run = int(run)
        args.run = int(run)
        bids_ses_label = yaxil.bids.legal.sub('', args.label)
        bids_sub_label = yaxil.bids.legal.sub('', subject_label)
        args.sub = 'sub-' + bids_sub_label
        args.ses = 'ses-' + bids_ses_label
        logger.debug('sub=%s, ses=%s', args.sub, args.ses)
        boldqc.cli.process.do(args)



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

requires = [
    'selfie',
    'executors',
    'yaxil',
    'matplotlib',
    'nibabel',
    'scipy'
]

test_requirements = [
]

about = dict()
with open(os.path.join(here, 'boldqc', '__version__.py'), 'r') as f:
    exec(f.read(), about)

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    packages=find_packages(),
    package_data={
        '': ['libexec/*.py', 'libexec/*.sh']
    },
    include_package_data=True,
    scripts=[
        'scripts/boldQC.py'
    ],
    install_requires=requires,
    tests_require=test_requirements
)
